@@ -1,16 +1,21 @@
 from typing import Dict, Optional, Sequence
+from functools import lru_cache
 from django.db import models
 from django.utils.translation import pgettext_lazy, pgettext
 
 from .query import TranslationQuerySet
 from .utils import get_language_choices
 from .conf import settings
+from .db import TranslatedRelationField
 
 
 __all__ = (
     'TRANSLATION_POSTFIX',
     'TRANSLATION_RELATED_NAME',
     'translated_model_uniques',
+    'base_translated_model',
+    'create_language_field',
+    'create_entity_field',
     'create_translated_model',
 )
 
@@ -25,13 +30,71 @@ def translated_model_uniques():
     )
 
 
-def collect_model_fields(model: models.Model, fields: Sequence[str]) -> Dict[str, models.Field]:
+def collect_model_fields(
+    model: models.Model, fields: Sequence[str]
+) -> Dict[str, models.Field]:
     return {
         field: model._meta.get_field(field).clone()
         for field in fields
     }
 
 
+@lru_cache
+def base_translated_model():
+    class BaseTranslatedModel(models.Model):
+        objects = TranslationQuerySet.as_manager()
+
+        class Meta:
+            abstract = True
+
+        id = models.BigAutoField(
+            primary_key=True, verbose_name=pgettext_lazy('pxd_lingua', 'ID')
+        )
+
+        def __str__(self):
+            return (
+                pgettext(
+                    'pxd_lingua',
+                    '"{language}" translation for #{entity_id} entity'
+                )
+                .format(language=self.language, entity_id=self.entity_id)
+            )
+
+    return BaseTranslatedModel
+
+
+def create_language_field(
+    default_language: Optional[str] = EMPTY,
+    languages: Optional[Sequence] = EMPTY
+):
+    return models.CharField(
+        verbose_name=pgettext_lazy('pxd_lingua', 'Language'),
+        max_length=32, choices=(
+            get_language_choices()
+            if languages is EMPTY
+            else languages
+        ),
+        default=(
+            settings.DEFAULT_LANGUAGE
+            if default_language is EMPTY
+            else default_language
+        ),
+        null=False, blank=False,
+    )
+
+
+def create_entity_field(
+    model: models.Model,
+    related_name: str = TRANSLATION_RELATED_NAME
+):
+    return models.ForeignKey(
+        model,
+        verbose_name=pgettext_lazy('pxd_lingua', 'Translation entity'),
+        null=False, blank=False, on_delete=models.CASCADE,
+        related_name=related_name
+    )
+
+
 def create_translated_model(
     model: models.Model,
     fields: Optional[Sequence[str]] = None,
@@ -62,44 +125,19 @@ def create_translated_model(
         if value
     })
 
-    class TranslatedModel(models.Model):
-        objects = TranslationQuerySet.as_manager()
-
+    class TranslatedModel(base_translated_model()):
         class Meta:
             abstract = True
 
-        id = models.BigAutoField(
-            primary_key=True, verbose_name=pgettext_lazy('pxd_lingua', 'ID')
-        )
-        language = models.CharField(
-            verbose_name=pgettext_lazy('pxd_lingua', 'Language'),
-            max_length=32, choices=(
-                get_language_choices()
-                if languages is EMPTY
-                else languages
-            ),
-            default=(
-                settings.DEFAULT_LANGUAGE
-                if default_language is EMPTY
-                else default_language
-            ),
-            null=False, blank=False,
-        )
-        entity = models.ForeignKey(
-            model,
-            verbose_name=pgettext_lazy('pxd_lingua', 'Translation entity'),
-            null=False, blank=False, on_delete=models.CASCADE,
-            related_name=related_name
+        language = create_language_field(
+            default_language=default_language,
+            languages=languages
         )
+        entity = create_entity_field(model, related_name=related_name)
 
-        def __str__(self):
-            return (
-                pgettext(
-                    'pxd_lingua',
-                    '"{language}" translation for #{entity_id} entity'
-                )
-                .format(language=self.language, entity_id=self.entity_id)
-            )
+        translation_reverse_relation = TranslatedRelationField(
+            model, on_delete=models.CASCADE, related_name='translated'
+        )
 
     return type(name, (TranslatedModel,), {
         **({} if no_fields else collect_model_fields(model, fields)),
